// © 2023 Daily, Co. All Rights Reserved

// ignore_for_file: invalid_annotation_target

import 'package:freezed_annotation/freezed_annotation.dart';

import '../util.dart';

part 'call_config.freezed.dart';
part 'call_config.g.dart';

enum RecordingMode {
  cloud('cloud'),
  rawTracks('raw-tracks'),
  local('local'),
  off('off');

  const RecordingMode(this.value);

  final String value;
}

enum StreamingEndpointType { hls, rtmp }

@freezed
class StreamingEndpoint with _$StreamingEndpoint {
  const factory StreamingEndpoint({
    required String name,
    required StreamingEndpointType type,
    required int config,
  }) = _StreamingEndpoint;

  factory StreamingEndpoint.fromJson(Map<String, dynamic> json) => _$StreamingEndpointFromJson(json);
}

DateTime? _dateTimeFromInt(int? timestamp) => timestamp?.let((it) => DateTime.fromMillisecondsSinceEpoch(it * 1000));

Duration? _durationFromInt(int? durationSeconds) => durationSeconds?.let((it) => Duration(seconds: it));

@freezed
class CallConfig with _$CallConfig {
  const factory CallConfig({
    required String roomName,
    String? initialUserName,
    String? userId,
    required bool isOwner,
    required bool initialCameraEnabled,
    required bool initialMicrophoneEnabled,
    required RecordingMode recordingMode,
    @JsonKey(fromJson: _dateTimeFromInt) DateTime? roomExpiration,
    @JsonKey(fromJson: _durationFromInt) Duration? tokenExpiration,
    @JsonKey(fromJson: _dateTimeFromInt) DateTime? roomNotBefore,
    @JsonKey(fromJson: _dateTimeFromInt) DateTime? tokenNotBefore,
    required bool ejectAtRoomExpiration,
    required bool ejectAtTokenExpiration,
    //permissionsOnJoin: ParticipantPermissions, todo: CSDK-657
    @Default([]) List<StreamingEndpoint> defaultStreamingEndpoints,
    required bool terseLoggingEnabled,
  }) = _CallConfig;

  factory CallConfig.fromJson(Map<String, dynamic> json) => _$CallConfigFromJson(json);
}
