// © 2022 Daily, Co. All Rights Reserved

import 'package:freezed_annotation/freezed_annotation.dart';

import '../../daily_flutter.dart';
import 'call_config.dart';

part 'call_state.freezed.dart';
part 'call_state.g.dart';

Map<String, CallState> _values = {for (final action in CallState.values) action.name: action};

/// A new [CallClient] instance starts in the [CallState.initialized] state.
/// As soon as [CallClient.join] is invoked, it progresses to the [CallState.joining] state, and when
/// the client has joined the call, it progresses further to the [CallState.joined] state.
/// As soon as [CallClient.leave] is invoked, the state changes to [CallState.leaving], followed by
/// [CallState.left] when this process has completed.
enum CallState {
  initialized,
  joining,
  joined,
  leaving,
  left;

  factory CallState.fromJson(String value) =>
      _values[value] ?? (throw ArgumentError('Unsupported CallState: "$value"'));
}

@Freezed(unionKey: 'state')
class CallStateData with _$CallStateData {
  const CallStateData._();

  const factory CallStateData.initialized() = _CallStateDataInitialized;

  const factory CallStateData.joining() = _CallStateDataJoining;

  const factory CallStateData.joined({
    // ignore: invalid_annotation_target
    @JsonKey(name: 'callConfiguration') required CallConfig callConfig,
  }) = _CallStateDataJoined;

  const factory CallStateData.leaving() = _CallStateDataLeaving;

  const factory CallStateData.left() = _CallStateDataLeft;

  factory CallStateData.fromJson(Map<String, dynamic> json) => _$CallStateDataFromJson(json);

  CallState get state => map(
        initialized: (_) => CallState.initialized,
        joining: (_) => CallState.joining,
        joined: (_) => CallState.joined,
        leaving: (_) => CallState.leaving,
        left: (_) => CallState.left,
      );
}

@freezed
class MeetingSession with _$MeetingSession {
  factory MeetingSession({
    String? id,
  }) = _MeetingSession;

  factory MeetingSession.fromJson(Map<String, dynamic> json) => _$MeetingSessionFromJson(json);
}

@freezed
class CallJoinData with _$CallJoinData {
  factory CallJoinData({
    required MeetingSession meetingSession,
  }) = _CallJoinData;

  factory CallJoinData.fromJson(Map<String, dynamic> json) => _$CallJoinDataFromJson(json);
}
