// © 2022 Daily, Co. All Rights Reserved

import 'package:collection/collection.dart';
import 'package:freezed_annotation/freezed_annotation.dart';

import 'devices.dart';
import 'update.dart';

part 'media.freezed.dart';
part 'media.g.dart';

@Freezed(copyWith: false)
class VideoMediaTrackSettings with _$VideoMediaTrackSettings {
  const factory VideoMediaTrackSettings({
    required DeviceId? deviceId,
    required int width,
    required int height,
    required double frameRate,
    required MediaTrackFacingMode facingMode,
  }) = _VideoMediaTrackSettings;

  factory VideoMediaTrackSettings.fromJson(Map<String, dynamic> json) => _$VideoMediaTrackSettingsFromJson(json);
}

@Freezed(toJson: true)
class VideoMediaTrackSettingsUpdate
    with _$VideoMediaTrackSettingsUpdate
    implements Update<VideoMediaTrackSettingsUpdate> {
  const factory VideoMediaTrackSettingsUpdate.set({
    Update<DeviceId>? deviceId,
    Update<int>? width,
    Update<int>? height,
    Update<double>? frameRate,
    MediaTrackFacingModeUpdate? facingMode,
  }) = _VideoMediaTrackSettingsUpdate;

  static const Update<VideoMediaTrackSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

enum MediaTrackFacingMode {
  user,
  environment;

  static MediaTrackFacingMode? fromString(String string) =>
      MediaTrackFacingMode.values.firstWhereOrNull((m) => m.name == string);

  MediaTrackFacingMode get flipped {
    switch (this) {
      case MediaTrackFacingMode.user:
        return MediaTrackFacingMode.environment;
      case MediaTrackFacingMode.environment:
        return MediaTrackFacingMode.user;
    }
  }

  MediaTrackFacingModeUpdate get asUpdate {
    switch (this) {
      case MediaTrackFacingMode.user:
        return MediaTrackFacingModeUpdate.user;
      case MediaTrackFacingMode.environment:
        return MediaTrackFacingModeUpdate.environment;
    }
  }
}

enum MediaTrackFacingModeUpdate { fromDefaults, user, environment }

@Freezed(copyWith: false)
class AudioMediaTrackSettings with _$AudioMediaTrackSettings {
  const factory AudioMediaTrackSettings({
    DeviceId? deviceId,
  }) = _AudioMediaTrackSettings;

  factory AudioMediaTrackSettings.fromJson(Map<String, dynamic> json) => _$AudioMediaTrackSettingsFromJson(json);
}

@Freezed(toJson: true)
class AudioMediaTrackSettingsUpdate
    with _$AudioMediaTrackSettingsUpdate
    implements Update<AudioMediaTrackSettingsUpdate> {
  const factory AudioMediaTrackSettingsUpdate.set({
    Update<DeviceId>? deviceId,
  }) = _AudioMediaTrackSettingsUpdate;

  static const Update<AudioMediaTrackSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

@freezed
class MediaTrackDeviceIdUpdate with _$MediaTrackDeviceIdUpdate implements Update<DeviceId> {
  const MediaTrackDeviceIdUpdate._();

  const factory MediaTrackDeviceIdUpdate.set({
    required DeviceId deviceId,
  }) = _MediaTrackDeviceIdUpdate;

  static const Update<DeviceId> fromDefaults = UpdateFromDefaults.instance;

  @override
  String toJson() => deviceId.id;
}
