// © 2022 Daily, Co. All Rights Reserved

import 'package:freezed_annotation/freezed_annotation.dart';

import 'update.dart';

part 'publishing.freezed.dart';
part 'publishing.g.dart';

@Freezed(copyWith: false)
class PublishingSettings with _$PublishingSettings {
  const factory PublishingSettings({
    required MicrophonePublishingSettings microphone,
    required CameraPublishingSettings camera,
  }) = _PublishingSettings;

  factory PublishingSettings.fromJson(Map<String, dynamic> json) => _$PublishingSettingsFromJson(json);
}

@Freezed(toJson: true)
class PublishingSettingsUpdate with _$PublishingSettingsUpdate implements Update<PublishingSettingsUpdate> {
  const factory PublishingSettingsUpdate.set({
    Update<MicrophonePublishingSettingsUpdate>? microphone,
    Update<CameraPublishingSettingsUpdate>? camera,
  }) = _PublishingSettingsUpdate;
  static const Update<PublishingSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

@Freezed(copyWith: false)
class MicrophonePublishingSettings with _$MicrophonePublishingSettings {
  const factory MicrophonePublishingSettings({
    required bool isPublishing,
    required AudioSendSettings sendSettings,
  }) = _MicrophonePublishingSettings;

  factory MicrophonePublishingSettings.fromJson(Map<String, dynamic> json) =>
      _$MicrophonePublishingSettingsFromJson(json);
}

@Freezed(toJson: true)
class MicrophonePublishingSettingsUpdate
    with _$MicrophonePublishingSettingsUpdate
    implements Update<MicrophonePublishingSettingsUpdate> {
  const factory MicrophonePublishingSettingsUpdate.set({
    Update<bool>? isPublishing,
    Update<VideoSendSettingsUpdate>? encodings,
  }) = _MicrophonePublishingSettingsUpdate;
  static const Update<MicrophonePublishingSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

@Freezed(copyWith: false)
class AudioSendSettings with _$AudioSendSettings {
  const factory AudioSendSettings({
    required AudioSendSettingsChannelConfig channelConfig,
    required int? bitrate,
  }) = _AudioSendSettings;

  factory AudioSendSettings.fromJson(Map<String, dynamic> json) => _$AudioSendSettingsFromJson(json);
}

@Freezed(toJson: true)
class AudioSendSettingsUpdate with _$AudioSendSettingsUpdate implements Update<AudioSendSettingsUpdate> {
  const factory AudioSendSettingsUpdate.set({
    AudioSendSettingsChannelConfigUpdate? channelConfig,
    Update<int>? bitrate,
  }) = _AudioSendSettingsUpdate;
  static const Update<AudioSendSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

enum AudioSendSettingsChannelConfig { mono, stereo }

enum AudioSendSettingsChannelConfigUpdate { fromDefaults, mono, stereo }

enum AudioSendQuality { speech, music }

enum AudioSendQualityUpdate { fromDefaults, speech, music }

@Freezed(copyWith: false)
class CameraPublishingSettings with _$CameraPublishingSettings {
  const factory CameraPublishingSettings({
    required bool isPublishing,
    required VideoSendSettings sendSettings,
  }) = _CameraPublishingSettings;

  factory CameraPublishingSettings.fromJson(Map<String, dynamic> json) => _$CameraPublishingSettingsFromJson(json);
}

@Freezed(toJson: true)
class CameraPublishingSettingsUpdate
    with _$CameraPublishingSettingsUpdate
    implements Update<CameraPublishingSettingsUpdate> {
  const factory CameraPublishingSettingsUpdate.set({
    Update<bool>? isPublishing,
    Update<VideoSendSettingsUpdate>? sendSettings,
  }) = _CameraPublishingSettingsUpdate;
  static const Update<CameraPublishingSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

@Freezed(copyWith: false)
class VideoSendSettings with _$VideoSendSettings {
  const factory VideoSendSettings({
    required VideoSendSettingsMaxQuality maxQuality,
    required VideoEncodingSettingsByQuality encodings,
  }) = _VideoSendSettings;

  factory VideoSendSettings.fromJson(Map<String, dynamic> json) => _$VideoSendSettingsFromJson(json);
}

@Freezed(toJson: true)
class VideoSendSettingsUpdate with _$VideoSendSettingsUpdate implements Update<VideoSendSettingsUpdate> {
  const factory VideoSendSettingsUpdate.set({
    VideoSendSettingsMaxQualityUpdate? maxQuality,
    Update<VideoEncodingSettingsByQualityUpdate>? encodings,
  }) = _VideoSendSettingsUpdate;
  static const Update<VideoSendSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

enum VideoSendSettingsMaxQuality { low, medium, high }

enum VideoSendSettingsMaxQualityUpdate { fromDefaults, low, medium, high }

@Freezed(copyWith: false)
class VideoEncodingSettingsByQuality with _$VideoEncodingSettingsByQuality {
  const factory VideoEncodingSettingsByQuality({
    required VideoEncodingSettings low,
    VideoEncodingSettings? medium,
    VideoEncodingSettings? high,
  }) = _VideoEncodingSettingsByQuality;

  factory VideoEncodingSettingsByQuality.fromJson(Map<String, dynamic> json) =>
      _$VideoEncodingSettingsByQualityFromJson(json);
}

@Freezed(toJson: true)
class VideoEncodingSettingsByQualityUpdate
    with _$VideoEncodingSettingsByQualityUpdate
    implements Update<VideoEncodingSettingsByQualityUpdate> {
  const factory VideoEncodingSettingsByQualityUpdate.set({
    Update<VideoEncodingSettingsUpdate>? low,
    Update<VideoEncodingSettingsUpdate>? medium,
    Update<VideoEncodingSettingsUpdate>? high,
  }) = _VideoEncodingSettingsByQualityUpdate;
  static const Update<VideoEncodingSettingsByQualityUpdate> fromDefaults = UpdateFromDefaults.instance;
}

@Freezed(copyWith: false)
class VideoEncodingSettings with _$VideoEncodingSettings {
  const factory VideoEncodingSettings({
    required int maxBitrate,
    required double maxFramerate,
    required double scaleResolutionDownBy,
  }) = _VideoEncodingSettings;

  factory VideoEncodingSettings.fromJson(Map<String, dynamic> json) => _$VideoEncodingSettingsFromJson(json);
}

@Freezed(toJson: true)
class VideoEncodingSettingsUpdate with _$VideoEncodingSettingsUpdate implements Update<VideoEncodingSettingsUpdate> {
  const factory VideoEncodingSettingsUpdate.set({
    Update<int>? maxBitrate,
    Update<double>? maxFrameRate,
    Update<double>? scaleResolutionDownBy,
  }) = _VideoEncodingSettingsUpdate;
  static const Update<VideoEncodingSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}
