// © 2022 Daily, Co. All Rights Reserved

import 'package:freezed_annotation/freezed_annotation.dart';

import 'participants.dart';
import 'update.dart';

part 'subscriptions.freezed.dart';
part 'subscriptions.g.dart';

@Freezed(toJson: false, fromJson: false)
class SubscriptionProfile with _$SubscriptionProfile {
  const factory SubscriptionProfile(String name) = _SubscriptionProfile;
  const SubscriptionProfile._();
  factory SubscriptionProfile.fromJson(String name) => SubscriptionProfile(name);

  static const SubscriptionProfile base = SubscriptionProfile('base');
  static const SubscriptionProfile activeRemote = SubscriptionProfile('active-remote');

  String toJson() => name;
}

@Freezed(toJson: false)
class SubscriptionProfileUpdate with _$SubscriptionProfileUpdate implements Update<SubscriptionProfile> {
  const SubscriptionProfileUpdate._();
  const factory SubscriptionProfileUpdate.set({required SubscriptionProfile profile}) = _SubscriptionProfileUpdate;
  static const Update<SubscriptionProfileUpdate> fromDefaults = UpdateFromDefaults.instance;

  @override
  String toJson() => profile.toJson();
}

@freezed
class SubscriptionProfileSettingsUpdatesByProfile
    with _$SubscriptionProfileSettingsUpdatesByProfile
    implements Update<SubscriptionProfileSettingsUpdatesByProfile> {
  const factory SubscriptionProfileSettingsUpdatesByProfile.set({
    required Map<SubscriptionProfile, MediaSubscriptionSettingsUpdate> updates,
  }) = _SubscriptionProfileSettingsUpdatesByProfile;
  const SubscriptionProfileSettingsUpdatesByProfile._();
  static Update<SubscriptionProfileSettingsUpdatesByProfile> get fromDefaults => UpdateFromDefaults.instance;

  @override
  Map<String, dynamic> toJson() => updates.map((key, value) => MapEntry(key.name, value.toJson()));
}

@freezed
class SubscriptionSettingsUpdatesById
    with _$SubscriptionSettingsUpdatesById
    implements Update<SubscriptionSettingsUpdatesById> {
  const factory SubscriptionSettingsUpdatesById.set({
    required Map<ParticipantId, SubscriptionSettingsUpdate> updates,
  }) = _SubscriptionSettingsUpdatesById;
  const SubscriptionSettingsUpdatesById._();

  static Update<SubscriptionSettingsUpdatesById> get fromDefaults => UpdateFromDefaults.instance;

  @override
  Map<String, dynamic> toJson() => updates.map((key, value) => MapEntry(key.id, value.toJson()));
}

Map<String, dynamic>? _subscriptionSettingsUpdatesByProfileToJson(
  Map<SubscriptionProfile, SubscriptionSettingsUpdate>? updates,
) =>
    updates?.map((key, value) => MapEntry(key.name, value.toJson()));

@freezed
class SubscriptionSettingsUpdatesByProfile
    with _$SubscriptionSettingsUpdatesByProfile
    implements Update<SubscriptionSettingsUpdatesByProfile> {
  const factory SubscriptionSettingsUpdatesByProfile.set({
    // ignore: invalid_annotation_target
    @JsonKey(toJson: _subscriptionSettingsUpdatesByProfileToJson)
        required Map<SubscriptionProfile, SubscriptionSettingsUpdate> updates,
  }) = _SubscriptionSettingsUpdatesByProfile;
  const SubscriptionSettingsUpdatesByProfile._();
  static Update<SubscriptionSettingsUpdatesByProfile> get fromDefaults => UpdateFromDefaults.instance;

  @override
  Map<String, dynamic> toJson() => updates.map((key, value) => MapEntry(key.name, value.toJson()));
}

@Freezed(copyWith: false)
class SubscriptionSettings with _$SubscriptionSettings {
  const factory SubscriptionSettings({
    required SubscriptionProfile profile,
    required MediaSubscriptionSettings media,
  }) = _SubscriptionSettings;

  factory SubscriptionSettings.fromJson(Map<String, dynamic> json) => _$SubscriptionSettingsFromJson(json);
}

@Freezed(toJson: true)
class SubscriptionSettingsUpdate with _$SubscriptionSettingsUpdate implements Update<SubscriptionSettingsUpdate> {
  const factory SubscriptionSettingsUpdate.set({
    Update<SubscriptionProfile>? profile,
    Update<MediaSubscriptionSettingsUpdate>? media,
  }) = _SubscriptionSettingsUpdate;
  static const Update<SubscriptionSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

@Freezed(copyWith: false)
class MediaSubscriptionSettings with _$MediaSubscriptionSettings {
  const factory MediaSubscriptionSettings({
    required CameraSubscriptionSettings camera,
    required MicrophoneSubscriptionSettings microphone,
  }) = _MediaSubscriptionSettings;

  factory MediaSubscriptionSettings.fromJson(Map<String, dynamic> json) => _$MediaSubscriptionSettingsFromJson(json);
}

@Freezed(toJson: true)
class MediaSubscriptionSettingsUpdate
    with _$MediaSubscriptionSettingsUpdate
    implements Update<MediaSubscriptionSettingsUpdate> {
  const factory MediaSubscriptionSettingsUpdate.set({
    Update<CameraSubscriptionSettingsUpdate>? camera,
    Update<MicrophoneSubscriptionSettingsUpdate>? microphone,
  }) = _MediaSubscriptionSettingsUpdate;
  static const Update<MediaSubscriptionSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

@Freezed(copyWith: false)
class MicrophoneSubscriptionSettings with _$MicrophoneSubscriptionSettings {
  const factory MicrophoneSubscriptionSettings({
    required SubscriptionState subscriptionState,
  }) = _MicrophoneSubscriptionSettingsValue;

  factory MicrophoneSubscriptionSettings.fromJson(Map<String, dynamic> json) =>
      _$MicrophoneSubscriptionSettingsFromJson(json);
}

@Freezed(toJson: true)
class MicrophoneSubscriptionSettingsUpdate
    with _$MicrophoneSubscriptionSettingsUpdate
    implements Update<MicrophoneSubscriptionSettingsUpdate> {
  const factory MicrophoneSubscriptionSettingsUpdate.set({
    SubscriptionStateUpdate? subscriptionState,
  }) = _MicrophoneSubscriptionSettingsUpdateValue;
  static const Update<MicrophoneSubscriptionSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

@Freezed(copyWith: false)
class CameraSubscriptionSettings with _$CameraSubscriptionSettings {
  const factory CameraSubscriptionSettings({
    required SubscriptionState subscriptionState,
    required VideoReceiveSettings receiveSettings,
  }) = _CameraSubscriptionSettingsValue;

  factory CameraSubscriptionSettings.fromJson(Map<String, dynamic> json) => _$CameraSubscriptionSettingsFromJson(json);
}

@Freezed(toJson: true)
class CameraSubscriptionSettingsUpdate
    with _$CameraSubscriptionSettingsUpdate
    implements Update<CameraSubscriptionSettingsUpdate> {
  const factory CameraSubscriptionSettingsUpdate.set({
    SubscriptionStateUpdate? subscriptionState,
    Update<VideoReceiveSettingsUpdate>? receiveSettings,
  }) = _CameraSubscriptionSettingsUpdateValue;
  static const Update<CameraSubscriptionSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

@Freezed(copyWith: false)
class VideoReceiveSettings with _$VideoReceiveSettings {
  const factory VideoReceiveSettings({
    required VideoReceiveSettingsMaxQuality maxQuality,
  }) = _VideoReceiveSettingsValue;

  factory VideoReceiveSettings.fromJson(Map<String, dynamic> json) => _$VideoReceiveSettingsFromJson(json);
}

@Freezed(toJson: true)
class VideoReceiveSettingsUpdate with _$VideoReceiveSettingsUpdate implements Update<VideoReceiveSettingsUpdate> {
  const factory VideoReceiveSettingsUpdate.set({
    VideoReceiveSettingsMaxQualityUpdate? maxQuality,
  }) = _VideoReceiveSettingsUpdateValue;
  static const Update<VideoReceiveSettingsUpdate> fromDefaults = UpdateFromDefaults.instance;
}

enum VideoReceiveSettingsMaxQuality { low, medium, high }

enum VideoReceiveSettingsMaxQualityUpdate { fromDefaults, low, medium, high }

enum SubscriptionState { subscribed, unsubscribed }

enum SubscriptionStateUpdate { fromDefaults, subscribed, unsubscribed }
